import std.string;
import std.process;

import file;
import utils;

class Config
{
    string USER_HOME_DIR, USER_DOTCONFIG_DIR;
    string OB_CFG_DIR, MENUXML, RCXML, AUTOSTARTSH;
    string OBMENUGEN_CFG_DIR, OBMENUGEN_CFG, OBMENUGEN_EXCLUSIONS_FILE;
    string[] DOT_DESKTOP_LOCATIONS = ["/usr/share/applications",
                                      "/opt/kde/share/applications/kde"];

    this()
    {        
        USER_HOME_DIR = std.process.getenv("HOME");
        //USER_HOME_DIR = "/tmp/fake_home"; //For using when developing
        USER_DOTCONFIG_DIR = USER_HOME_DIR ~ "/.config/";
        
        OB_CFG_DIR = USER_DOTCONFIG_DIR ~ "openbox/";
        MENUXML = OB_CFG_DIR ~ "menu.xml";
        RCXML = OB_CFG_DIR ~ "rc.xml";
        AUTOSTARTSH = OB_CFG_DIR ~ "autostart.sh";
        OBMENUGEN_CFG_DIR = USER_DOTCONFIG_DIR ~ "obmenugen/";
        OBMENUGEN_CFG = OBMENUGEN_CFG_DIR ~ "obmenugen.cfg";
        OBMENUGEN_EXCLUSIONS_FILE = OBMENUGEN_CFG_DIR ~ "exclusions";        
        
        guaranteeConfigFilesExists();
        
        parseConfigFile();
    }
    
    
    string[] getKeys() { return ConfigPairs.keys.sort; }
    
    
    void showWindowsAndDesktopMenu(bool show)
    {
        if (show)
            ConfigPairs["show windows and desktops menu"] = "true";
        else
            ConfigPairs["show windows and desktops menu"] = "false";
    }
    
    
    string getValue(string key)
    {
        try
        {
            return ConfigPairs[key];
        }
        catch (Exception e)
        {
            string msg = format("Configuration error: Key '%s' not found in config file.", key);
            throw new Exception(msg);
        }
    }
    
    
    private:
        string[string] ConfigPairs;


        void parseConfigFile()
        {
            foreach(uint i, line; File.Load(OBMENUGEN_CFG).Lines())
            {
                if (isComment(line)) continue;
                
                string[] pair = split(line, "=");
                
                if (pair.length < 2)
                {
                    string msg = format("Configuration error: Line #%d isn't a valid 'key = value' pair.", i+1);
                    throw new Exception(msg);
                }
                
                foreach(uint idx, item; pair)
                    pair[idx] = pair[idx].getClean();
                
                if (pair[1].length == 0) pair[1] = "none";
                
                ConfigPairs[pair[0]] = pair[1];
            }                
        }
                
        
        void guaranteeConfigFilesExists()
        { 
            if (!std.file.exists("/etc/xdg/openbox/rc.xml"))
            {
                string msg = "Error: I can't find the file '/etc/xdg/openbox/rc.xml'\n"
                "Have you OpenBox installed?\n"
                "If you have the file in your system, simply copy it to '~/.config/openbox/'\n"
                "If you feel this is a bug, please, contact me in [neonskull at gmail.com]'";
                throw new Exception(msg);
            }
            
            if (!std.file.exists(USER_DOTCONFIG_DIR))
                std.file.mkdir(USER_DOTCONFIG_DIR);
            
            if (!std.file.exists(OB_CFG_DIR))
                std.file.mkdir(OB_CFG_DIR);
            
            if (!(std.file.exists(OBMENUGEN_CFG_DIR)))
                std.file.mkdir(OBMENUGEN_CFG_DIR);
            
            if (!std.file.exists(AUTOSTARTSH))
            {
                File.Write(AUTOSTARTSH, "#!/bin/bash\n");
                string cmd = std.string.format("chmod 0744 %s", AUTOSTARTSH);
                system(cmd);
            }
            
            if (!std.file.exists(RCXML)) std.file.copy("/etc/xdg/openbox/rc.xml", RCXML);
            
            if (!std.file.exists(OBMENUGEN_CFG))
            {
                string cfg = "# OpenBox Menu Generator config file\n#\n"
                "# Lines starting with a '#' character and empty lines are ignored.\n"
                "# Keys and values are case sensitive. Keep all keys lowercase.\n"
                "# Removing or commenting any config line will cause an error.\n\n"
                "# Top Menu section\n"
                "# 'terminal' and 'editor' are REQUIRED\n"
                "# 'filemanager' and 'run command' are optional.\n"
                "# Leaving optional values empty causes the corresponding\n"
                "# menu item to be hidden, no matter what show/hide option says.\n"
                "terminal = xterm\n"
                "editor = xterm -e nano\n"
                "filemanager = thunar\n"
                "run command = gmrun\n"
                "\n# Show/Hide Top Menu Items\n"
                "show terminal item = true\n"
                "show editor item = true\n"
                "show filemanager item = true\n"
                "show run command item = true\n"
                "\n# Show/Hide 'Applications' Section label\n"
                "show apps section label = true\n"
                "\n# Lock Screen menuitem\n"
                "# Also leaving it empty causes menuitem to be hidden\n"
                "lock command = xscreensaver-command -lock\n"
                "\n# Exit menuitem\n"
                "# Giving this an empty value don't hides the menuitem\n"
                "# but uses the default OpenBox action \"Exit\"\n"
                "exit command = \n"
                "\n# OnlyShowIn Exclusions\n"
                "# There is a 'OnlyShowIn' directive in .desktop files.\n"
                "# This directive is self-descriptive :)\n"
                "# You can put here (in lowercase) any directive's posible value\n"
                "# you want to be excluded from the menu, comma-separeted.\n"
                "# Leaving this value empty won't exclude any entry by this criteria.\n"
                "exclude onlyshowin = kde,gnome\n"
                "\n";
                
                File.Write(OBMENUGEN_CFG, cfg);
            }
            
            if (!std.file.exists(OBMENUGEN_EXCLUSIONS_FILE))
            {
                string exclusions = "# OpenBox Menu Generator exclusions file\n#\n"
                "# Lines starting with a '#' character and empty lines are ignored.\n"
                "# Put one exclusion per line.\n"
                "# Exclusions are case sensitive and are matched against\n"
                "# the 'Name' of the entry (the text that appears in menuitems).\n"
                "#\n# Exclusions format examples:\n"
                "# '^example$' exact match with 'example'.\n"
                "# '^example'  match any string that begins whith 'example'.\n"
                "# 'example$'  match any string that ends with 'example'.\n"
                "# 'example'   match any string containing 'example'.\n"
                "\n";
                
                File.Write(OBMENUGEN_EXCLUSIONS_FILE, exclusions);
            }
            
        }
}



/*int main(string[] argv)
{
    try
    {
        Config cfg = new Config();
        writefln(cfg.getValue("exit"));
    }
    catch (Exception e)
    {
        writefln(e.msg);
        return 1;
    }
    
    return 0;
}
*/
